import { Request, Response } from 'express';
import { LoginUserReq, UserEntity } from '../types';
import { UserRecord } from '../records/user.record';
import { CustomError } from '../utils/handleError';
import { compare } from 'bcrypt';
import { sign, verify } from 'jsonwebtoken';
import { config } from '../config/config';

export class UsersController {
  static async register(req: Request, res: Response) {
    const userReq = req.body as UserEntity;
    const user = new UserRecord(userReq);
    await user.insert();
    res.status(201).json(user.email);
  }

  static async login(req: Request, res: Response) {
    const loginUserReq = req.body as LoginUserReq;
    if (!loginUserReq.email || !loginUserReq.password) {
      throw new CustomError('Email and password are required.', 400);
    }
    const user = await UserRecord.getOneByEmail(loginUserReq.email);
    if (!user) throw new CustomError('Wrong email or password.', 400);
    const match = await compare(loginUserReq.password, user.password);
    if (!match) throw new CustomError('Wrong email or password.', 400);

    const refreshToken = sign({ id: user.id }, config.REFRESH_TOKEN_SECRET, { expiresIn: '365d' });
    const accessToken = sign({ id: user.id }, config.ACCESS_TOKEN_SECRET, { expiresIn: '15min' });

    user.refreshToken = refreshToken;
    await user.update();

    res.cookie('__refresh', refreshToken, {
      httpOnly: true,
      secure: true,
      sameSite: 'none',
      maxAge: 365 * 24 * 60 * 60 * 1000,
    });
    res.status(200).json({ id: user.id, username: user.username, accessToken });
  }

  static async logout(req: Request, res: Response) {
    const { id } = req.body;
    if (!id) throw new Error('Id not sent from frontend app.');
    const user = await UserRecord.getOneById(id);
    user.refreshToken = null;
    await user.update();
    res.clearCookie('__refresh', { httpOnly: true, sameSite: 'none', secure: true });
    res.sendStatus(200);
  }

  static async refresh(req: Request, res: Response) {
    const refreshToken = req.cookies['__refresh'];
    if (!refreshToken) throw new CustomError('Unauthorized', 401)
    const { id } = verify(refreshToken, config.REFRESH_TOKEN_SECRET) as { id: string };
    const user = await UserRecord.getOneById(id);
    if (user?.refreshToken !== refreshToken) throw new CustomError('Forbidden', 403);
    const accessToken = sign({ id }, config.ACCESS_TOKEN_SECRET, { expiresIn: '15min' });
    res.status(200).json(accessToken);
  }
}