"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserRecord = void 0;
const db_1 = require("../utils/db");
const uuid_1 = require("uuid");
const types_1 = require("../types");
const handleError_1 = require("../utils/handleError");
const bcrypt_1 = require("bcrypt");
const USER_REGEX = /^[a-zA-Z][a-zA-Z0-9-_]{3,24}$/;
class UserRecord {
    constructor(obj) {
        var _a, _b;
        this.id = obj.id;
        this.username = (_a = obj.username) === null || _a === void 0 ? void 0 : _a.trim();
        this.email = (_b = obj.email) === null || _b === void 0 ? void 0 : _b.trim();
        this.password = obj.password;
        this.role = obj.role || types_1.UserRole.USER;
        this.refreshToken = obj.refreshToken;
        this.validate();
    }
    validate() {
        if (!this.username) {
            throw new handleError_1.CustomError('Username is required.');
        }
        if (!this.email) {
            throw new handleError_1.CustomError('Email is required.');
        }
        if (!this.password) {
            throw new handleError_1.CustomError('Password is required.');
        }
        if (this.username.length < 4 || this.username.length > 25) {
            throw new handleError_1.CustomError('Username must be longer than 3 character and shorter than 26 charaters.');
        }
        if (!/^[a-zA-Z]$/.test(this.username[0])) {
            throw new handleError_1.CustomError('Username must begins with letter.');
        }
        if (!USER_REGEX.test(this.username)) {
            throw new handleError_1.CustomError('Username can contains only letters, numbers, "_" and "-"');
        }
        if (this.email.length > 254 || this.email.length < 3 || !this.email.includes('@')) {
            throw new handleError_1.CustomError('Email is not valid. Type correct email.');
        }
        if (this.password.length < 6) {
            throw new handleError_1.CustomError('Password must be equal or longer than 6 characters.');
        }
        if (this.password.length > 100) {
            throw new handleError_1.CustomError('Password is to long. Maximum characters for password is 100.');
        }
    }
    insert() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.id = (0, uuid_1.v4)();
                this.refreshToken = null;
                this.password = yield (0, bcrypt_1.hash)(this.password, yield (0, bcrypt_1.genSalt)(10));
                console.log(this);
                yield db_1.pool.execute('INSERT INTO `users` VALUES(:id, :username, :email, :password, :role, :refreshToken)', this);
                return this.id;
            }
            catch (e) {
                console.log(e);
                const message = e.message.toLowerCase();
                if (message.includes('duplicate')) {
                    const duplicateKey = message.split(' ')[5].replaceAll('\'', '');
                    throw new handleError_1.CustomError(`That ${duplicateKey} is already taken.`, 409);
                }
            }
        });
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            yield db_1.pool.execute('UPDATE `users` SET `refreshToken` = :refreshToken, `username` = :username, `email` = :email WHERE `id` = :id', this);
        });
    }
    static getOneByEmail(email) {
        return __awaiter(this, void 0, void 0, function* () {
            const [users] = yield db_1.pool.execute('SELECT * FROM `users` WHERE `email` = :email', {
                email,
            });
            return users.length === 0 ? null : new UserRecord(users[0]);
        });
    }
    static getOneById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const [users] = yield db_1.pool.execute('SELECT * FROM `users` WHERE `id` = :id', {
                id,
            });
            return users.length === 0 ? null : new UserRecord(users[0]);
        });
    }
}
exports.UserRecord = UserRecord;
//# sourceMappingURL=user.record.js.map