import { AdEntity, AdEntityWithUsername, SmallAdEntity } from '../types';
import { CustomError } from '../utils/handleError';
import { v4 as uuid } from 'uuid';
import { pool } from '../utils/db';
import { FieldPacket } from 'mysql2';

export class AdRecord implements AdEntity {
  id?: string;
  userId: string;
  username?: string;
  title: string;
  description: string;
  price: number;
  url: string;
  lat: number;
  lng: number;
  createdAt?: Date;

  constructor(obj: AdEntityWithUsername) {
    this.id = obj.id;
    this.userId = obj.userId;
    this.title = obj.title?.trim();
    this.description = obj.description?.trim();
    this.price = obj.price;
    this.url = obj.url;
    this.lat = obj.lat;
    this.lng = obj.lng;
    this.createdAt = obj.createdAt;
    this.username = obj.username;
    this.validate();
  }

  private validate() {
    if (!this.title) {
      throw new CustomError('Title is required');
    }
    if (this.price === undefined) {
      throw new CustomError('Price is required');
    }
    if (!this.url) {
      throw new CustomError('Link is required');
    }
    if (this.title.length < 5 || this.title.length > 70) {
      throw new CustomError('Title must be longer than 5 characters and shorter than 70.');
    }
    if (this.description.length > 300) {
      throw new CustomError('Description cannot be longer than 300 characters.');
    }
    if (typeof this.price !== 'number') {
      throw new CustomError('Price must be a number.');
    }
    if (this.price < 0) {
      throw new CustomError('Price cannot be lower than 0.');
    }
    if (this.price > 99999999) {
      throw new CustomError('I think that You want too much for your thing.');
    }
    if (this.url.length > 500) {
      throw new CustomError('Link cannot be longer than 500 characters.');
    }
  }

  async insert(): Promise<string> {
    this.id = uuid();
    this.createdAt = new Date();
    await pool.execute('INSERT INTO `ads` VALUES(:id, :userId, :title, :description, :price, :url, :lat, :lng, :createdAt)', this);
    return this.id;
  }

  async update(): Promise<void> {
    await pool.execute('UPDATE `ads` SET `title` = :title, `description` = :description, `price` = :price, `url` = :url WHERE `id` = :id', this);
  }

  static async searchAll(term: string): Promise<SmallAdEntity[]> {
    const [res] = await pool.execute('SELECT `id`, `userId`, `lat`, `lng` FROM `ads` WHERE `title` LIKE :term', {
      term: `%${term}%`,
    }) as [SmallAdEntity[], FieldPacket[]];
    return res;
  }

  static async getOneById(id: string): Promise<AdRecord | null> {
    const [res] = await pool.execute('SELECT `ads`.`id`, `ads`.`userId`,`ads`.`title`,`ads`.`description`,`ads`.`price`,`ads`.`url`,`ads`.`lat`,`ads`.`lng`,`ads`.`createdAt`, `users`.`username`  FROM `ads` JOIN `users` ON `ads`.`userId` = `users`.`id` WHERE `ads`.`id` = :id', { id }) as [AdEntityWithUsername[], FieldPacket[]];
    return res.length === 0 ? null : new AdRecord(res[0]);
  }

  static async deleteOne(id: string): Promise<void> {
    await pool.execute('DELETE FROM `ads` WHERE `id` = :id', { id });
  }
}