import { pool } from '../utils/db';
import { v4 as uuid } from 'uuid';
import { UserEntity, UserRole } from '../types';
import { CustomError } from '../utils/handleError';
import { genSalt, hash } from 'bcrypt';
import { FieldPacket } from 'mysql2';

const USER_REGEX = /^[a-zA-Z][a-zA-Z0-9-_]{3,24}$/;
type UserDatabaseResponse = [UserEntity[], FieldPacket[]];

export class UserRecord implements UserEntity {
  id?: string;
  username: string;
  email: string;
  password: string;
  role: UserRole;
  refreshToken?: string;

  constructor(obj: UserEntity) {
    this.id = obj.id;
    this.username = obj.username?.trim();
    this.email = obj.email?.trim();
    this.password = obj.password;
    this.role = obj.role || UserRole.USER;
    this.refreshToken = obj.refreshToken;
    this.validate();
  }

  private validate() {
    if (!this.username) {
      throw new CustomError('Username is required.');
    }
    if (!this.email) {
      throw new CustomError('Email is required.');
    }
    if (!this.password) {
      throw new CustomError('Password is required.');
    }
    if (this.username.length < 4 || this.username.length > 25) {
      throw new CustomError('Username must be longer than 3 character and shorter than 26 charaters.');
    }
    if (!/^[a-zA-Z]$/.test(this.username[0])) {
      throw new CustomError('Username must begins with letter.');
    }
    if (!USER_REGEX.test(this.username)) {
      throw new CustomError('Username can contains only letters, numbers, "_" and "-"');
    }
    if (this.email.length > 254 || this.email.length < 3 || !this.email.includes('@')) {
      throw new CustomError('Email is not valid. Type correct email.');
    }
    if (this.password.length < 6) {
      throw new CustomError('Password must be equal or longer than 6 characters.');
    }
    if (this.password.length > 100) {
      throw new CustomError('Password is to long. Maximum characters for password is 100.');
    }
  }

  async insert(): Promise<string> {
    try {
      this.id = uuid();
      this.refreshToken = null;
      this.password = await hash(this.password, await genSalt(10));
      console.log(this);
      await pool.execute('INSERT INTO `users` VALUES(:id, :username, :email, :password, :role, :refreshToken)', this);
      return this.id;
    } catch (e) {
      console.log(e);
      const message = e.message.toLowerCase();
      if (message.includes('duplicate')) {
        const duplicateKey = message.split(' ')[5].replaceAll('\'', '');
        throw new CustomError(`That ${ duplicateKey } is already taken.`, 409);
      }
    }
  }

  async update(): Promise<void> {
    await pool.execute('UPDATE `users` SET `refreshToken` = :refreshToken, `username` = :username, `email` = :email WHERE `id` = :id', this);
  }

  static async getOneByEmail(email: string): Promise<UserRecord | null> {
    const [users] = await pool.execute('SELECT * FROM `users` WHERE `email` = :email', {
      email,
    }) as UserDatabaseResponse;
    return users.length === 0 ? null : new UserRecord(users[0]);
  }

  static async getOneById(id: string): Promise<UserRecord | null> {
    const [users] = await pool.execute('SELECT * FROM `users` WHERE `id` = :id', {
      id,
    }) as UserDatabaseResponse;
    return users.length === 0 ? null : new UserRecord(users[0]);
  }
}